Package: HCAData
Type: Package
Title: Accessing The Datasets Of The Human Cell Atlas in R/Bioconductor
Version: 1.29.0
Authors@R: c(
    person("Federico","Marini", role=c("aut","cre"), email="marinif@uni-mainz.de",
    comment = c(ORCID = '0000-0003-3252-7758')))
Description: This package allows a direct access to the dataset generated by the 
    Human Cell Atlas project for further processing in R and Bioconductor, in the
    comfortable format of SingleCellExperiment objects
    (available in other formats here: http://preview.data.humancellatlas.org/).
License: MIT + file LICENSE
Encoding: UTF-8
biocViews: RNASeqData, SingleCellData, ExperimentData, ExpressionData,
        ExperimentHub
Depends: R (>= 3.6), SingleCellExperiment
Imports: ExperimentHub, AnnotationHub, HDF5Array, utils
Suggests: knitr, rmarkdown, BiocStyle, scran (>= 1.11.4), BiocSingular,
        scater, scuttle, Rtsne, igraph, iSEE, testthat
URL: https://github.com/federicomarini/HCAData
VignetteBuilder: knitr
RoxygenNote: 7.3.2
Roxygen: list(markdown = TRUE)
git_url: https://git.bioconductor.org/packages/HCAData
git_branch: devel
git_last_commit: b37a62e
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-30
NeedsCompilation: no
Packaged: 2026-04-30 15:42:47 UTC; biocbuild
Author: Federico Marini [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3252-7758>)
Maintainer: Federico Marini <marinif@uni-mainz.de>

# Generated by roxygen2: do not edit by hand

export(HCAData)
importFrom(AnnotationHub,query)
importFrom(ExperimentHub,ExperimentHub)
importFrom(ExperimentHub,createHubAccessors)
importFrom(HDF5Array,HDF5Array)
importFrom(SingleCellExperiment,SingleCellExperiment)
importFrom(utils,read.csv)

<!-- badges: start -->

[![R build
status](https://github.com/federicomarini/HCAData/workflows/R-CMD-check/badge.svg)](https://github.com/federicomarini/HCAData/actions)
<!-- badges: end -->

# HCAData

Available at <http://bioconductor.org/packages/HCAData/>.

This is an
R/[ExperimentHub](https://bioconductor.org/packages/release/bioc/html/ExperimentHub.html)
package for accessing data from the [Human Cell
Atlas](https://www.humancellatlas.org) as
[`SingleCellExperiment`](https://bioconductor.org/packages/release/bioc/html/SingleCellExperiment.html)
objects.

The datasets can otherwise be downloaded from the [Human Cell Atlas Data
Portal](https://preview.data.humancellatlas.org), where they are
available to the research community, either in raw format (with scripts
to download the primary data), or in hdf5-based format (for the sets
generated via the 10x method).

If you use the data, please refer to the original sources as well (plus
the related publications, which will be listed here when they will be
out), which are licensed under a Creative Commons Attribution 4.0
International License.

## Some additional information

-   The
    [`cellranger`](https://support.10xgenomics.com/single-cell-gene-expression/software/pipelines/latest/what-is-cell-ranger)-processed
    data files and metadata were downloaded from the links provided in
    [the Data Portal](https://preview.data.humancellatlas.org) on
    November 5, 2018

-   This current release includes the following datasets:

| dataset.ID             | subset               | species      | technology | cell.number | description                                                                                 |
|:-----------------------|:---------------------|:-------------|:-----------|------------:|:--------------------------------------------------------------------------------------------|
| Census of Immune Cells | Umbilical cord blood | Homo sapiens | 10x        |      384000 | Profiling of immunocytes by single cell RNA-seq for understanding human health and disease. |
| Census of Immune Cells | Bone marrow          | Homo sapiens | 10x        |      378000 | Profiling of immunocytes by single cell RNA-seq for understanding human health and disease. |

-   Since the hdf5 files are derived from the cellranger pipeline, some
    information on their specific format can be found
    [here](https://support.10xgenomics.com/single-cell-gene-expression/software/pipelines/latest/advanced/h5_matrices)

## Some new dataset has been released but it is not in this package!

In this case, feel free to contact the author(s) of this package, and/or
to provide a complete pull request with the recipe to add the missing
dataset.

To get initial guidance for this, please refer to the excellent
vignettes of the
[ExperimentHub](https://bioconductor.org/packages/release/bioc/html/ExperimentHub.html)
package.

## Code of Conduct

Please note that the HCAData project is released with a [Contributor
Code of
Conduct](https://contributor-covenant.org/version/2/0/CODE_OF_CONDUCT.html).
By contributing to this project, you agree to abide by its terms.

#' @title Download data from the HCA via ExperimentHub
#'
#' @description Download HDF5 (dense assay) and RDS (row and column annotations)
#' files from the HCA via ExperimentHub, composing them together as a
#' SingleCellExperiment object
#'
#' @details This current release includes the following datasets:
#' \describe{
#'   \item{Census of Immune Cells - Umbilical cord blood}{UMI counts from the 10x
#'   (droplet) single-cell RNA-seq data. The object contains counts for 33694 genes
#'   in 384000 cells.}
#'   \item{Census of Immune Cells - Bone marrow}{UMI counts from the 10x
#'   (droplet) single-cell RNA-seq data. The object contains counts for 33694 genes
#'   in 378000 cells.}
#' }
#'
#' @param dataset A character string: which dataset should be retrieved?
#' @param as.sparse Logical, specifies whether the underlying HDF5 dataset should
#' be treated as sparse or not - will be passed to the call to `HDF5Array()`.
#' Defaults to TRUE, i.e. by using the `DelayedArray` infrastructure.
#'
#' @return A SingleCellExperiment object with a HDF5Matrix in the `counts`
#' assay, containing the UMI counts for each gene in each cell. Row- and column-level
#' metadata are also provided.
#'
#' @export
#'
#' @examples
#' HCAData()
#' sce_cordblood <- HCAData("ica_cord_blood")
HCAData <- function(dataset = NULL, as.sparse = TRUE) {
  available_datasets <- c("ica_bone_marrow", "ica_cord_blood")
  if(!is.null(dataset)) {
    if(!(dataset %in% available_datasets)) {
      stop("No dataset found with the specified name, please choose one of the following: \n",
           paste(available_datasets,collapse = "\n"))
    }
  } else {
    message("No dataset name provided, please choose one of the following: \n",
            paste(available_datasets,collapse = "\n"))
    return(invisible(NULL))
  }

  message("Retrieving Human Cell Atlas dataset: ", dataset)
  hub <- ExperimentHub::ExperimentHub()
  base <- "HCAData/"

  ## row and column data
  rdatapath <- paste0(base, dataset, "_rowData.rds")
  rowData <- query(hub, rdatapath)[[1]]

  suppressMessages({
    cdatapath <- paste0(base, dataset, "_colData.rds")
    colData <- query(hub, cdatapath)[[1]]

    ## HDF5, from ExperimentHub:
    h5datapath <- paste0(base, dataset, "_rectangular.h5")
    h5file <- query(hub, h5datapath)[[1]]
  })
  h5array <- HDF5Array(h5file, "counts", as.sparse = as.sparse)

  sce <- SingleCellExperiment(
    list(counts = h5array),
    rowData = rowData,
    colData = colData
  )
  rownames(sce) <- rowData(sce)$ID
  colnames(sce) <- colData(sce)$Barcode
  return(sce)
}

context("sce objects are correct")

test_that("empty call", {
  library(HCAData)
  expect_message(HCAData())
})

test_that("wrong call to the dataset", {
  library(HCAData)
  expect_error(HCAData("cordblood"))
})

test_that("cord blood data is correct", {
  library(HCAData)
  sce_cordblood <- HCAData("ica_cord_blood")
  expect_is(sce_cordblood, "SingleCellExperiment")
  expect_equal(nrow(sce_cordblood), 33694)
  expect_equal(ncol(sce_cordblood), 384000)
  expect_equal(colnames(rowData(sce_cordblood)), c("ID", "Symbol"))
  expect_equal(assayNames(sce_cordblood), "counts")
  expect_equal(length(grep("^ERCC-", rownames(sce_cordblood))), 0)
})

test_that("bone marrow data is correct", {
  library(HCAData)
  sce_bonemarrow <- HCAData("ica_bone_marrow")
  expect_is(sce_bonemarrow, "SingleCellExperiment")
  expect_equal(nrow(sce_bonemarrow), 33694)
  expect_equal(ncol(sce_bonemarrow), 378000)
  expect_equal(colnames(rowData(sce_bonemarrow)), c("ID", "Symbol"))
  expect_equal(assayNames(sce_bonemarrow), "counts")
  expect_equal(length(grep("^ERCC-", rownames(sce_bonemarrow))), 0)
})

## validate with `ExperimentHubData::makeExperimentHubMetadata()`
## (above pkg directory)

df_HCA_cordblood <- data.frame(
  Title = paste0("Human Cell Atlas - Census of Immune Cells, Umbilical cord blood, ",
                 c("'dense matrix' format","sample (column) annotation","gene (row) annotation")),
  Description = paste0("Census of Immune Cells, Umbilical cord blood - 10x data from the Human Cell Atlas project, profiling of immunocytes by single cell RNA-seq for understanding human health and disease. Represented as HDF5 count matrix and serialized rowData and colData objects, derived from the raw counts matrix on https://preview.data.humancellatlas.org - ",
                       c("Full rectangular, block-compressed format, 1GB block size.",
                         "Sample descriptions (barcodes).",
                         "Gene descriptions.")),
  BiocVersion = rep("3.9", 3),
  Genome = rep("GRCh38", 3),
  SourceType = rep("HDF5", 3),
  SourceUrl = rep("https://s3.amazonaws.com/preview-ica-expression-data/ica_cord_blood_h5.h5", 3),
  SourceVersion = rep("b5dd8ec9c9ff2d142986aaee014b9098", 3),
  Species = rep("Homo sapiens", 3),
  TaxonomyId = rep(9606, 3),
  Coordinate_1_based = rep(TRUE, 3),
  DataProvider = rep("Human Cell Atlas", 3),
  Maintainer = rep("Federico Marini <marinif@uni-mainz.de>", 3),
  RDataClass = rep("character", 3),
  DispatchClass = c("H5File","Rds","Rds"),
  RDataPath = paste0("HCAData/ica_cord_blood",c("_rectangular.h5", "_colData.rds", "_rowData.rds")),
  stringsAsFactors = FALSE
)

df_HCA_bonemarrow <- data.frame(
  Title = paste0("Human Cell Atlas - Census of Immune Cells, Bone marrow, ",
                 c("'dense matrix' format","sample (column) annotation","gene (row) annotation")),
  Description = paste0("Census of Immune Cells, Bone marrow - 10x data from the Human Cell Atlas project, profiling of immunocytes by single cell RNA-seq for understanding human health and disease. Represented as HDF5 count matrix and serialized rowData and colData objects, derived from the raw counts matrix on https://preview.data.humancellatlas.org - ",
                       c("Full rectangular, block-compressed format, 1GB block size.",
                         "Sample descriptions (barcodes).",
                         "Gene descriptions.")),
  BiocVersion = rep("3.9", 3),
  Genome = rep("GRCh38", 3),
  SourceType = rep("HDF5", 3),
  SourceUrl = rep("https://s3.amazonaws.com/preview-ica-expression-data/ica_bone_marrow_h5.h5", 3),
  SourceVersion = rep("9d10973b7c178ee6886ae12d004d4d23", 3),
  Species = rep("Homo sapiens", 3),
  TaxonomyId = rep(9606, 3),
  Coordinate_1_based = rep(TRUE, 3),
  DataProvider = rep("Human Cell Atlas", 3),
  Maintainer = rep("Federico Marini <marinif@uni-mainz.de>", 3),
  RDataClass = rep("character", 3),
  DispatchClass = c("H5File","Rds","Rds"),
  RDataPath = paste0("HCAData/ica_bone_marrow",c("_rectangular.h5", "_colData.rds", "_rowData.rds")),
  stringsAsFactors = FALSE
)

# when there's going to be many more, it might be practical to have it automated

## Combined meta-data
df_all <- do.call(rbind,lapply(ls(pattern = "^df_HCA"),get))

## Save .csv file
write.csv(df_all, file = "../extdata/metadata.csv", row.names = FALSE)

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  error = FALSE,
  warning = FALSE,
  message = FALSE
)

## ----install, eval=FALSE------------------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE))
#   install.packages("BiocManager")
# BiocManager::install("HCAData")

## ----loadpkg------------------------------------------------------------------
library("HCAData")

## ----firstcall----------------------------------------------------------------
HCAData()

## ----fetchingdata-------------------------------------------------------------
suppressPackageStartupMessages({
  library("ExperimentHub")
  library("SingleCellExperiment")
})

eh <- ExperimentHub()
query(eh, "HCAData")

# these three are the components to the bone marrow dataset
bonemarrow_h5densematrix <- eh[["EH2047"]]
bonemarrow_coldata <- eh[["EH2048"]]
bonemarrow_rowdata <- eh[["EH2049"]]

# and are put together when calling...
sce_bonemarrow <- HCAData("ica_bone_marrow")
sce_bonemarrow

# similarly, to access the umbilical cord blood dataset
sce_cordblood <- HCAData("ica_cord_blood")
sce_cordblood

## ----subset-------------------------------------------------------------------
library("scran")
library("BiocSingular")
library("scater")
library("scuttle")

set.seed(42)
sce <- sce_bonemarrow[, sample(seq_len(ncol(sce_bonemarrow)), 1000, replace = FALSE)]

## ----featureanno--------------------------------------------------------------
rownames(sce) <- uniquifyFeatureNames(rowData(sce)$ID, rowData(sce)$Symbol)
head(rownames(sce))

is.mito <- grep("MT-", rownames(sce))

counts(sce) <- as.matrix(counts(sce))
sce <- scuttle::addPerCellQC(sce, subsets=list(Mito=is.mito))

## ----clustnorm----------------------------------------------------------------
lib.sf.bonemarrow <- librarySizeFactors(sce)
summary(lib.sf.bonemarrow)

set.seed(42)
clusters <- quickCluster(sce)
table(clusters)
sce <- computeSumFactors(sce, min.mean=0.1, cluster=clusters)
sce <- logNormCounts(sce)
assayNames(sce)

## ----hvg----------------------------------------------------------------------
dec.bonemarrow <- modelGeneVarByPoisson(sce)
top.dec <- dec.bonemarrow[order(dec.bonemarrow$bio, decreasing=TRUE),] 
head(top.dec)

fit.bonemarrow <- metadata(dec.bonemarrow)
plot(fit.bonemarrow$mean, fit.bonemarrow$var, xlab="Mean of log-expression",
    ylab="Variance of log-expression", pch = 16)
curve(fit.bonemarrow$trend(x), col="dodgerblue", add=TRUE, lwd=2)
plotExpression(sce, features=rownames(top.dec)[1:10])

## ----dimred-------------------------------------------------------------------
hvg.bonemarrow <- getTopHVGs(dec.bonemarrow, prop = 0.1)

set.seed(42)
sce <- denoisePCA(sce, 
                  technical=dec.bonemarrow, 
                  subset.row = hvg.bonemarrow, 
                  BSPARAM=IrlbaParam())
ncol(reducedDim(sce, "PCA"))
plot(attr(reducedDim(sce), "percentVar"), xlab="PC",
     ylab="Proportion of variance explained")
abline(v=ncol(reducedDim(sce, "PCA")), lty=2, col="red")
plotPCA(sce, ncomponents=3, colour_by="subsets_Mito_percent")

set.seed(42)
sce <- runTSNE(sce, dimred="PCA", perplexity=30)
plotTSNE(sce, colour_by="subsets_Mito_percent")

## ----clusters-----------------------------------------------------------------
snn.gr <- buildSNNGraph(sce, use.dimred="PCA")
clusters <- igraph::cluster_walktrap(snn.gr)
sce$Cluster <- factor(clusters$membership)
table(sce$Cluster)
plotTSNE(sce, colour_by="Cluster")

## ----launchisee, eval=FALSE---------------------------------------------------
# if (require(iSEE)) {
#   iSEE(sce)
# }

## ----save, eval=FALSE---------------------------------------------------------
# destination <- "where/to/store/the/processed/data.rds"
# saveRDS(sce, file = destination)

## ----sessioninfo--------------------------------------------------------------
sessionInfo()

#' HCAData: Accessing The Datasets Of The Human Cell Atlas in R/Bioconductor
#'
#' `HCAData` is an ExperimentHub package which provides access to the single-cell
#' RNA-seq data from the Human Cell Atlas project (<https://www.humancellatlas.org>)
#'
#' @references If you use the data in this package, please refer to the original sources
#' (Human Cell Atlas Data Portal, <https://preview.data.humancellatlas.org>) as well
#' (plus the related publications, which will be listed here when they will be out),
#' which are licensed under a Creative Commons Attribution 4.0 International License.
#'
#' @importFrom utils read.csv
#' @importFrom ExperimentHub createHubAccessors ExperimentHub
#' @importFrom AnnotationHub query
#' @importFrom HDF5Array HDF5Array
#' @importFrom SingleCellExperiment SingleCellExperiment
#'
#' @author Federico Marini \email{marinif@@uni-mainz.de}
#' @name HCAData-pkg
#' @keywords internal
"_PACKAGE"

---
title: "Download and preprocess the Human Cell Atlas bone marrow data (Census of Immune Cells)"
author: "Federico Marini"
date: ""
output: html_document
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
options(width = 100)
```

# Helper functions

Following the definitions provided [here](https://support.10xgenomics.com/single-cell-gene-expression/software/pipelines/latest/advanced/h5_matrices)...

```{r}
library(HDF5Array)
library(SingleCellExperiment)

getGenomes <- function(path) {
  genomes <- attributes(h5dump(path, load = FALSE))$names
  genomes
}

getRowData <- function(path, genome = NULL) {
  if (is.null(genome)) 
    genome <- getGenomes(path)[1]
  data.frame(
    ID = as.character(h5read(path, paste0(genome,"/genes"))),
    Symbol=as.character(h5read(path, paste0(genome,"/gene_names"))),
    stringsAsFactors=FALSE
  )
}

getColData <- function(path, genome = NULL) {
  if (is.null(genome)) 
    genome <- getGenomes(path)[1]
  
  barcode <- as.character(h5read(path, paste0(genome,"/barcodes")))
  
  data.frame(
    Barcode=barcode,
    stringsAsFactors=FALSE
  )
}
```

# Downloading the h5 file

(accessed November 6, 2018)

```{r}
url <- "https://s3.amazonaws.com/preview-ica-expression-data/ica_bone_marrow_h5.h5"
path <- basename(url)
if(!file.exists(path))
  download.file(url, path)
```

# Loading the data and preprocessing

Saving `rowData` and `colData`, used when constructing the `SingleCellExperiment` object

```{r}
saveRDS(getRowData(path), "ica_bone_marrow_rowData.rds")
saveRDS(getColData(path), "ica_bone_marrow_colData.rds")
```

Converting into a `HDF5Matrix` object

```{r}
mygenome <- getGenomes("ica_bone_marrow_h5.h5")
tenxmat <- TENxMatrix(path, group = mygenome)
dim(tenxmat)
pryr::object_size(tenxmat
```

Writing out the object as a rectangular h5 file

```{r}
options(DelayedArray.block.size=1e9) # 1GB block size.
mat.out <- writeHDF5Array(
  tenxmat,
  file="ica_bone_marrow_rectangular.h5",
  name="counts",
  chunkdim=beachmat::getBestChunkDims(dim(tenxmat))
)
```

# Generating and saving the `SingleCellExperiment` object

```{r}
sce_bonemarrow <- 
  SingleCellExperiment(
    list(counts = HDF5Array("ica_bone_marrow_rectangular.h5", "counts")), 
    rowData = getRowData(path), 
    colData = getColData(path)
  )

rownames(sce_bonemarrow) <- rowData(sce_bonemarrow)$ID
colnames(sce_bonemarrow) <- colData(sce_bonemarrow)$Barcode
sce_bonemarrow

saveRDS(sce_bonemarrow, file = "sce_HCA_bonemarrow.rds")
```

# Session info

```{r}
sessionInfo()
```

---
title: "Accessing the Human Cell Atlas datasets"
author:
- name: Federico Marini
  affiliation: 
  - &id1 Institute of Medical Biostatistics, Epidemiology and Informatics (IMBEI), Mainz
  - Center for Thrombosis and Hemostasis (CTH), Mainz
  email: marinif@uni-mainz.de
date: "`r BiocStyle::doc_date()`"
package: "`r BiocStyle::pkg_ver('HCAData')`"
output: 
  BiocStyle::html_document:
    toc_float: true
vignette: >
  %\VignetteIndexEntry{Accessing the Human Cell Atlas datasets}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  %\VignettePackage{HCAData}
---

**Compiled date**: `r Sys.Date()`

**Last edited**: 2019-05-24

**License**: `r packageDescription("HCAData")[["License"]]`

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  error = FALSE,
  warning = FALSE,
  message = FALSE
)
```

# Introducing the `HCAData` package

The `r Biocpkg("HCAData")` package allows a direct access to the dataset generated by the Human Cell Atlas project for further processing in R and Bioconductor.
It does so by providing the datasets as `r Biocpkg("SingleCellExperiment")` objects, i.e. a format which is both efficient and very widely adopted throughout many existing Bioconductor workflows.

The datasets are otherwise available in other formats (also as raw data) at this link: http://preview.data.humancellatlas.org/.

# Installing `HCAData`

The `r Biocpkg("HCAData")` package can be installed in the conventional way via `r CRANpkg("BiocManager")`.

```{r install, eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
  install.packages("BiocManager")
BiocManager::install("HCAData")
```

This package makes extensive use of the `r Biocpkg("HDF5Array")` package to avoid loading the entire data set in memory.
Instead, it stores the counts on disk as a HDF5 file, and loads subsets of the data into memory upon request.

# Loading `HCAData` and the Human Cell Atlas data

```{r loadpkg}
library("HCAData")
```

We use the `HCAData` function to download the relevant files from Bioconductor's `r Biocpkg("ExperimentHub")` web resource. 
If no argument is provided, a list of the available datasets is returned, specifying which name to enter as `dataset` parameter when calling `HCAData`.

```{r firstcall}
HCAData()
```

The list of relevant files includes the HDF5 file containing the counts, as well as the metadata on the rows (genes) and columns (cells). 

The output is a single `SingleCellExperiment` object from the `r Biocpkg("SingleCellExperiment")` package. 

Being based on `r Biocpkg("ExperimentHub")`, the data related to this package can be accessed and queried directly using the package name. 
Retrieval is then as easy as using their ExperimentHub accession numbers (for the single components of each set), or by using the convenience function provided in this package.

```{r fetchingdata}
suppressPackageStartupMessages({
  library("ExperimentHub")
  library("SingleCellExperiment")
})

eh <- ExperimentHub()
query(eh, "HCAData")

# these three are the components to the bone marrow dataset
bonemarrow_h5densematrix <- eh[["EH2047"]]
bonemarrow_coldata <- eh[["EH2048"]]
bonemarrow_rowdata <- eh[["EH2049"]]

# and are put together when calling...
sce_bonemarrow <- HCAData("ica_bone_marrow")
sce_bonemarrow

# similarly, to access the umbilical cord blood dataset
sce_cordblood <- HCAData("ica_cord_blood")
sce_cordblood
```

# Explore data with `iSEE`

The datasets are provided in the form of a `SingleCellExperiment` object. 
A natural companion to this data structure is the `r Biocpkg("iSEE")` package, which can be used for interactive and reproducible data exploration.

Any analysis steps should be performed in advance before calling `iSEE`, and since these datasets can be quite big, the operations can be time consuming, and/or require a considerable amount of resources.

## Processing a subset of the HCA bone marrow data

For the scope of the vignette, we subset some cells in the bone marrow dataset to reduce the runtime, and apply some of the steps one would ideally follow in analysing droplet based datasets.
For more information on how to properly process such datasets, please refer to the amazing set of resources available in the `r Biocpkg("simpleSingleCell")` workflow package.

In brief: we start loading the required libraries for preprocessing, and taking a subset of the bone marrow dataset

```{r subset}
library("scran")
library("BiocSingular")
library("scater")
library("scuttle")

set.seed(42)
sce <- sce_bonemarrow[, sample(seq_len(ncol(sce_bonemarrow)), 1000, replace = FALSE)]
```

First, we relabel the rows with the gene symbols for easier reading with the `uniquifyFeatureNames()` function from `r Biocpkg("scater")`.
Then we compute some QC metrics and add these to the original `sce` object, using `addPerCellQC()`.

```{r featureanno}
rownames(sce) <- uniquifyFeatureNames(rowData(sce)$ID, rowData(sce)$Symbol)
head(rownames(sce))

is.mito <- grep("MT-", rownames(sce))

counts(sce) <- as.matrix(counts(sce))
sce <- scuttle::addPerCellQC(sce, subsets=list(Mito=is.mito))
```

We proceed with normalization, performed with the deconvolution method implemented in `r Biocpkg("scran")` - a pre-clustering step is done in advance, to avoid pooling cells that are very different between each other.

```{r clustnorm}
lib.sf.bonemarrow <- librarySizeFactors(sce)
summary(lib.sf.bonemarrow)

set.seed(42)
clusters <- quickCluster(sce)
table(clusters)
sce <- computeSumFactors(sce, min.mean=0.1, cluster=clusters)
sce <- logNormCounts(sce)
assayNames(sce)
```

In the following lines of code, we model the mean-variance trend (with technical noise as Poisson), to extract the biological component of the variance for the genes under inspection.

```{r hvg}
dec.bonemarrow <- modelGeneVarByPoisson(sce)
top.dec <- dec.bonemarrow[order(dec.bonemarrow$bio, decreasing=TRUE),] 
head(top.dec)

fit.bonemarrow <- metadata(dec.bonemarrow)
plot(fit.bonemarrow$mean, fit.bonemarrow$var, xlab="Mean of log-expression",
    ylab="Variance of log-expression", pch = 16)
curve(fit.bonemarrow$trend(x), col="dodgerblue", add=TRUE, lwd=2)
plotExpression(sce, features=rownames(top.dec)[1:10])
```

With the `denoisePCA` function from `r Biocpkg("scran")`, we can compute a reduced dimension view of the data, accounting for the Poisson technical trend.

Once we computed the PCA, we provide that as initialization to `runTSNE`, and obtain the t-SNE results, stored in the appropriate slot of our `sce` object. 

```{r dimred}
hvg.bonemarrow <- getTopHVGs(dec.bonemarrow, prop = 0.1)

set.seed(42)
sce <- denoisePCA(sce, 
                  technical=dec.bonemarrow, 
                  subset.row = hvg.bonemarrow, 
                  BSPARAM=IrlbaParam())
ncol(reducedDim(sce, "PCA"))
plot(attr(reducedDim(sce), "percentVar"), xlab="PC",
     ylab="Proportion of variance explained")
abline(v=ncol(reducedDim(sce, "PCA")), lty=2, col="red")
plotPCA(sce, ncomponents=3, colour_by="subsets_Mito_percent")

set.seed(42)
sce <- runTSNE(sce, dimred="PCA", perplexity=30)
plotTSNE(sce, colour_by="subsets_Mito_percent")
```

After this, we can compute a shared nearest neighbour graph to identify clusters in our dataset.
Once the cluster memberships are defined, we assign this vector to a corresponding `colData` slot, and plot the t-SNE for our subset, coloured accordingly.

```{r clusters}
snn.gr <- buildSNNGraph(sce, use.dimred="PCA")
clusters <- igraph::cluster_walktrap(snn.gr)
sce$Cluster <- factor(clusters$membership)
table(sce$Cluster)
plotTSNE(sce, colour_by="Cluster")
```

Even if we only took a subset of the available full data, we can observe that different clusters are indeed nicely separated.
Subsequent steps would then involve identification of marker genes, and more advanced downstream techniques, which are not part of the scope of this vignette. 
To read more on what can be done, the vignettes of the `r Biocpkg("simpleSingleCell")` workflow package are an excellent place to start.

## Exploring the dataset with `iSEE`

Once the processing steps above are done, we can call `iSEE` with the subsampled `SingleCellExperiment` object. 

```{r launchisee, eval=FALSE}
if (require(iSEE)) {
  iSEE(sce)
}
```

## Saving the processed object

You can save the `sce` object to a serialized R object with 

```{r save, eval=FALSE}
destination <- "where/to/store/the/processed/data.rds"
saveRDS(sce, file = destination)
```

The object can be read into a new R session with `readRDS(destination)`, provided the HDF5 file remains in its original location (conveniently stored in the default location of `ExperimentHub`).

# Session info {-}

```{r sessioninfo}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HCAData.R
\name{HCAData}
\alias{HCAData}
\title{Download data from the HCA via ExperimentHub}
\usage{
HCAData(dataset = NULL, as.sparse = TRUE)
}
\arguments{
\item{dataset}{A character string: which dataset should be retrieved?}

\item{as.sparse}{Logical, specifies whether the underlying HDF5 dataset should
be treated as sparse or not - will be passed to the call to \code{HDF5Array()}.
Defaults to TRUE, i.e. by using the \code{DelayedArray} infrastructure.}
}
\value{
A SingleCellExperiment object with a HDF5Matrix in the \code{counts}
assay, containing the UMI counts for each gene in each cell. Row- and column-level
metadata are also provided.
}
\description{
Download HDF5 (dense assay) and RDS (row and column annotations)
files from the HCA via ExperimentHub, composing them together as a
SingleCellExperiment object
}
\details{
This current release includes the following datasets:
\describe{
\item{Census of Immune Cells - Umbilical cord blood}{UMI counts from the 10x
(droplet) single-cell RNA-seq data. The object contains counts for 33694 genes
in 384000 cells.}
\item{Census of Immune Cells - Bone marrow}{UMI counts from the 10x
(droplet) single-cell RNA-seq data. The object contains counts for 33694 genes
in 378000 cells.}
}
}
\examples{
HCAData()
sce_cordblood <- HCAData("ica_cord_blood")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HCAData-pkg.R
\docType{package}
\name{HCAData-pkg}
\alias{HCAData-package}
\alias{HCAData-pkg}
\title{HCAData: Accessing The Datasets Of The Human Cell Atlas in R/Bioconductor}
\description{
\code{HCAData} is an ExperimentHub package which provides access to the single-cell
RNA-seq data from the Human Cell Atlas project (\url{https://www.humancellatlas.org})
}
\references{
If you use the data in this package, please refer to the original sources
(Human Cell Atlas Data Portal, \url{https://preview.data.humancellatlas.org}) as well
(plus the related publications, which will be listed here when they will be out),
which are licensed under a Creative Commons Attribution 4.0 International License.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/federicomarini/HCAData}
}

}
\author{
Federico Marini \email{marinif@uni-mainz.de}
}
\keyword{internal}
